@@ -119,7 +119,7 @@ gpgme_op_decrypt_result (gpgme_ctx_t ctx)
 
 
 static gpgme_error_t
-parse_enc_to (char *args, gpgme_recipient_t *recp)
+parse_enc_to (char *args, gpgme_recipient_t *recp, gpgme_protocol_t protocol)
 {
   gpgme_recipient_t rec;
   char *tail;
@@ -146,7 +146,7 @@ parse_enc_to (char *args, gpgme_recipient_t *recp)
   if (*args != '\0' && *args != ' ')
     {
       free (rec);
-      return gpg_error (GPG_ERR_INV_ENGINE);
+      return trace_gpg_error (GPG_ERR_INV_ENGINE);
     }
 
   while (*args == ' ')
@@ -155,12 +155,12 @@ parse_enc_to (char *args, gpgme_recipient_t *recp)
   if (*args)
     {
       gpg_err_set_errno (0);
-      rec->pubkey_algo = strtol (args, &tail, 0);
+      rec->pubkey_algo = _gpgme_map_pk_algo (strtol (args, &tail, 0), protocol);
       if (errno || args == tail || *tail != ' ')
 	{
 	  /* The crypto backend does not behave.  */
 	  free (rec);
-	  return gpg_error (GPG_ERR_INV_ENGINE);
+	  return trace_gpg_error (GPG_ERR_INV_ENGINE);
 	}
     }
 
@@ -261,7 +261,7 @@ _gpgme_decrypt_status_handler (void *priv, gpgme_status_code_t code,
       break;
 
     case GPGME_STATUS_ENC_TO:
-      err = parse_enc_to (args, opd->last_recipient_p);
+      err = parse_enc_to (args, opd->last_recipient_p, ctx->protocol);
       if (err)
 	return err;
 
@@ -283,7 +283,7 @@ _gpgme_decrypt_status_handler (void *priv, gpgme_status_code_t code,
 	  }
 	/* FIXME: Is this ok?  */
 	if (!rec)
-	  return gpg_error (GPG_ERR_INV_ENGINE);
+	  return trace_gpg_error (GPG_ERR_INV_ENGINE);
       }
       break;
 
@@ -377,7 +377,7 @@ gpgme_op_decrypt_start (gpgme_ctx_t ctx, gpgme_data_t cipher,
 
   if (!ctx)
     return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
-  
+
   err = decrypt_start (ctx, 0, cipher, plain);
   return TRACE_ERR (err);
 }
@@ -395,7 +395,7 @@ gpgme_op_decrypt (gpgme_ctx_t ctx, gpgme_data_t cipher, gpgme_data_t plain)
 
   if (!ctx)
     return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
-  
+
   err = decrypt_start (ctx, 1, cipher, plain);
   if (!err)
     err = _gpgme_wait_one (ctx);
